/*!
 * Copyright (C) 2002 by Call Direct Cellular Solutions Pty. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
  *    This product includes software developed by Call Direct Cellular Solutions Pty. Ltd.
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY CALL DIRECT CELLULAR SOLUTIONS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CALL DIRECT
 * CELLULAR SOLUTIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.calldirect.com.au/
 */

/*!
 * $Log: CDCS_Inet.c,v $
 * Revision 1.1  2002/10/29 12:39:36  harald
 * First check in
 *
 *
 * Revision 0.1  2002/06/21 MJC 
 * Created
 *
 */


#include <string.h>
#include <sys/print.h>			/* Formatted output functions. */
#include <sys/ifstream.h>		/* Stream IO functions */
#include <net/PPPif.h>
#include <netinet/sostream.h>
#include <arpa/inet.h>
#include <sys/device.h>

#include "CDCS_Inet.h"
#include "UltraLite_E47.h"

#ifdef NUTDEBUG
#include <sys/kprint.h>
#include <sys/osdebug.h>
#include <net/netdebug.h>
#endif

u_long gethostbyname (char *pName)
{
	UDPSOCKET	*sock;
	u_char		msg[128];
	u_char		*p;
	u_char		*o;
	u_short		n,t;
	u_short		port;
	u_char		i;
	u_long		addr;
	IFNET		*nif;


	// Create Socket
	sock = NutUdpCreateSocket(DNS_CLIENTPORT);

	// Get DNS server address
	nif = devPPP0.dev_icb;
	addr = nif->if_pdns;

	// Build query
	n = htons(0x0001);				//Identification
	memcpy(msg,&n,sizeof(n));
	n = htons(0x0100);				//Flags
	memcpy(msg+2,&n,sizeof(n));
	n = htons(0x0001);				//Number of questions = 1
	memcpy(msg+4,&n,sizeof(n));
	memset(msg+6,0,6);				//RRs = 0
	
	n = 12;
	p = pName;
	while ((o = (unsigned char *)strchr(p,'.')))
	{
		i = o - p;
		msg[n++] = i;
		memcpy(msg+n,p,i);
		n += i;
		p = o+1;
	}
	i = ((u_char *)(pName + strlen(pName))) - p;
	if (i)
	{
		msg[n++] = i;
		memcpy(msg+n,p,i);
		n += i;
	}
	msg[n++] = 0;
	p = msg+n;
	i = n+4;

	n = htons(0x0001);			//Type
	memcpy(p,&n,sizeof(n));	
	memcpy(p+2,&n,sizeof(n));	//class
	
	// Send request
	NutUdpSendTo(sock, addr, DNS_CLIENTPORT, msg, i);

	// Clear address
	addr = 0;

	// Get response
	if ( (n = NutUdpReceiveFrom(sock, &addr, &port, msg, 128, 5000)) >0)
	{
		//NutKPrintFormat("\r\nDNS n = %d ", n);
		memcpy(&n, msg+6,sizeof(n));
		n = ntohs(n);
		while (n--)
		{
			// Handle DNS name compression
			// Cause it's a name and we don't really care about names
			// Just skip it
			if (msg[i] & 0xC0)
			{
				i ++;
			}
			else while (msg[i])
			{
				i += msg[i];
				i ++;
				// if i > bytes received return failure
			}
			i ++;
			memcpy(&t,msg+i,2);				// Get the record type
			t = ntohs(t);
			//NutKPrintFormat("\r\nRR Type = %x ", t);
			if (t == 1)
			{
				i += 10;					// Just skip the details for now
				memcpy (&addr, msg+i, 4);	// Extract the address
				i += 4;	
				NutUdpDestroySocket (sock);	// Free up the socket 
				return addr;
			}
			else
			{
				i += 8;						// Just skip the details for now
				memcpy(&t,msg+i,2);			// Get the payload length
				t = ntohs(t);
				t += 2;						// Plus length of t
				//NutKPrintFormat("DLen = %x", t);
				i += t;						// And skip it
			}	
		}
	}
	NutUdpDestroySocket (sock);	// Free up the socket 
	return 0;
}

