/*!
 * Copyright (C) 2002 by Call Direct Cellular Solutions Pty. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
  *    This product includes software developed by Call Direct Cellular Solutions Pty. Ltd.
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY CALL DIRECT CELLULAR SOLUTIONS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CALL DIRECT
 * CELLULAR SOLUTIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.calldirect.com.au/
 */

/*!
 * $Log: CDCS_MMI.c,v $
 * Revision 1.1  2002/10/29 12:39:37  harald
 * First check in
 *
 *
 * Revision 0.1  2002/05/16 MJC 
 * Created
 *
 */

#include <string.h>
#include <sys/thread.h>    
#include <sys/print.h>      /* Formatted output functions. */
#include <sys/ifstream.h>	  /* Stream IO functions */
#include <sys/timer.h>
#include <net/if_var.h>
#include <sys/heap.h>
#include <sys/confos.h>
#include <arpa/inet.h>
#include <sys/device.h>
#include <net/PPPif.h>

#include "UltraLite_E47.h"
#include "CDCS_MMI.h"
#include "CDCS_Inet.h"
#include "CDCS_Telnet.h"

#ifdef NUTDEBUG
#include <sys/kprint.h>
#include <sys/osdebug.h>
#include <net/netdebug.h>
#endif

void MMI_Trace(NUTDEVICE *stream, u_char *args)
{
#ifdef NUTDEBUG
	if (tcp_trace)
	{
		NutKTcpTrace(0);
		NutPrintString_P(stream, PSTR("\r\nTCP Trace OFF\r\n"));
	}
	else
	{
		NutKTcpTrace(1);
		NutPrintString_P(stream, PSTR("\r\nTCP Trace ON\r\n"));
	}
#endif
}

void MMI_NSLookup(NUTDEVICE *stream, u_char *args)
{
	NutPrintFormat(stream, "\r\nName: %s", args);
	NutPrintFlush(stream);
	NutPrintFormat(stream, "\r\nAddress: %s\r\n",
		inet_ntoa(gethostbyname(args)));
} 

void MMI_Connect(NUTDEVICE *stream, u_char *args)
{
	Flags |= Auto_GPRS_Attach;
	PPPTerminateSesion(&devPPP0);
	NutPrintString_P(stream, PSTR("\r\nResetting PPP link\r\n"));
}


void MMI_IfConfig(NUTDEVICE *stream, u_char *args)
{
	NUTDEVICE	*dev;
	IFNET		*ifn;
	int			i;

	dev = nutDeviceList;
	while (dev)
	{
		NutPrintString_P(stream, PSTR("\r\n"));
		NutPrintString(stream,  dev->dev_name);
		NutPrintString_P(stream, PSTR("\tLink encap:"));
		if (dev->dev_type == IFTYP_STREAM) 
		{
			NutPrintString_P(stream, PSTR("None"));
		}
		if (dev->dev_type == IFTYP_NET)
		{
			ifn = dev->dev_icb;
			if (ifn->if_type == IFT_PPP)
			{
				NutPrintString_P(stream, PSTR("Point-Point Protocol\tLinkDevice: "));
				if (ifn->if_host_dev == NULL)
				{
					NutPrintString_P(stream, PSTR("Disabled"));
				}

				NutPrintString_P(stream, PSTR("\r\n\tInet addr: "));
				NutPrintString(stream, inet_ntoa(ifn->if_local_ip));

				NutPrintString_P(stream, PSTR("\tP-t-P: "));
				NutPrintString(stream, inet_ntoa(ifn->if_remote_ip));

				NutPrintString_P(stream, PSTR("\tMask: "));
				NutPrintString(stream, inet_ntoa(ifn->if_mask));
			}
			if (ifn->if_type == IFT_ETHER)
			{
				NutPrintString_P(stream, PSTR("Ethernet\t\tHWaddr "));
				for (i = 0; i < 6; i ++)
				{
					NutPrintInteger(stream, ifn->if_mac[i], 16, 2, (FMTFLG_ZERO | FMTFLG_CAPITAL));
					if (i<5)
						NutPrintString_P(stream, PSTR(":"));
				}
				NutPrintString_P(stream, PSTR("\r\n\tInet addr: "));
				NutPrintString(stream, inet_ntoa(ifn->if_local_ip));

				// Broadcast isn't supported YET
				//NutPrintString_P(stream, PSTR("\tBcast: "));
				//NutPrintString(stream, inet_ntoa(ifn->));

				NutPrintString_P(stream, PSTR("\tMask: "));
				NutPrintString(stream, inet_ntoa(ifn->if_mask));
			}
			NutPrintString_P(stream, PSTR("\r\n\tMTU: "));
			NutPrintInteger(stream, ifn->if_mtu, 10, 0, 0);
		}

		//
		// Print Inet Addr 
		//NutPrintString_P(stream, PSTR("\tBcast:"));
		// Pring bcast
		//NutPrintString_P(stream, PSTR("\tMask:"));
		// Print mask
		// UP BROADCAST RUNNING MULTICAST 
		//NutPrintString_P(stream, PSTR("\r\n\tMTU:"));
		// Print MTU
		// Print stats

		
		// UP RUNNING MULTICAST POINTOPOINT NOARP 
		//NutPrintString_P(stream, PSTR("\r\n\tMTU:"));
		// Print MTU
		// Print Stats

		//Link encap:Point-Point Protocol
		//inet addr:10.144.153.104  P-t-P:10.144.153.51 Mask:255.255.255.0
		NutPrintString_P(stream, PSTR("\r\n"));
		dev = dev->dev_next;
	}
         
}


void MMI_ShowThreadState(NUTDEVICE *stream, u_char *args)
{
	NUTTHREADINFO *tdp;
	NUTTIMERINFO *tnp;

	NutPrintString_P(stream, PSTR("\r\nList of threads with name,state,prio,stack,mem,timeout\r\n"));
	for(tdp = nutThreadList; tdp; tdp = tdp->td_next) 
	{
		NutPrintString(stream, tdp->td_name);
		NutPrintString_P(stream, PSTR("\t"));
		switch(tdp->td_state) 
		{
		case TDS_TERM:
			NutPrintString_P(stream, PSTR("\tTerm\t"));
			break;
		case TDS_RUNNING:
			NutPrintString_P(stream, PSTR("\tRun\t"));
			break;
		case TDS_READY:
			NutPrintString_P(stream, PSTR("\tReady\t"));
			break;
		case TDS_SLEEP:
			NutPrintString_P(stream, PSTR("\tSleep\t"));
			break;
		}
		NutPrintInteger(stream, tdp->td_priority, 10, 0, 0);
		NutPrintString_P(stream, PSTR("\t"));
		NutPrintInteger(stream, (u_short)tdp->td_sp - (u_short)tdp->td_memory, 10, 0, 0);
		if(*((u_long *)tdp->td_memory) != DEADBEEF)
			NutPrintString_P(stream, PSTR("\tCorrupted\t"));
		else
			NutPrintString_P(stream, PSTR("\tOK\t"));
		if((tnp = (NUTTIMERINFO *)tdp->td_timer) != 0) 
			NutPrintInteger(stream, tnp->tn_ticks_left, 10, 0, 0);
		else
			NutPrintString_P(stream, PSTR("None"));
		NutPrintString_P(stream, PSTR("\r\n"));
	}
}

void MMI_ShowFreeMem (NUTDEVICE *stream, u_char *args)
{
	NutPrintString_P(stream, PSTR("\r\n"));
	NutPrintInteger(stream, NutHeapAvailable(), 10, 0, 0);
	NutPrintString_P(stream, PSTR(" bytes RAM free\r\n"));
}

void MMI_ShowTimers (NUTDEVICE *stream, u_char *args)
{
	NUTTIMERINFO *tnp;

	NutPrintString_P(stream, PSTR("\r\nList of timers with ticks left and interval\r\n"));
	for(tnp = nutTimerList; tnp; tnp = tnp->tn_next) 
	{
		NutPrintInteger(stream, tnp->tn_ticks_left, 10, 0, 0);
		if(tnp->tn_ticks) 
		{
			NutPrintString_P(stream, PSTR("\t"));
			NutPrintInteger(stream, tnp->tn_ticks, 10, 0, 0);
		}
		else 
			NutPrintString_P(stream, PSTR("\tOneshot"));
		NutPrintString_P(stream, PSTR("\r\n"));
	}
}

//
// MMI Command definitions 
// To add new commands you need to: -
//	- Define the command text and optionally its description text and help text 
//  - Create a handler function for the new command
//  - Add a new entry to the MMICommands array
//
// AFIK You can't just define the text strings in the array definition as there doesn't
// seem to be a mechanism to allow this for program space data.
//
const char PROGMEM c1[] = "memory";
const char PROGMEM d1[] = "Queries number of RAM bytes free";

const char PROGMEM c2[] = "threads";
const char PROGMEM d2[] = "Lists all threads";

const char PROGMEM c3[] = "timers";
const char PROGMEM d3[] = "Lists all active timers";

const char PROGMEM c4[] = "telnet";
const char PROGMEM d4[] = "Establish interactive connection to a remote TCP Service";

const char PROGMEM c5[] = "connect";
const char PROGMEM d5[] = "Start GPRS session";

const char PROGMEM c6[] = "lookup";
const char PROGMEM d6[] = "Get the IP address for the given domain name";

const char PROGMEM c7[] = "trace";
const char PROGMEM d7[] = "Activate TCP debug trace";

const char PROGMEM c8[] = "tserver";
const char PROGMEM d8[] = "Start local terminal server";

const char PROGMEM c9[] = "ifconfig";
const char PROGMEM d9[] = "Show status of or configure Serial PPP and Ethernet interfaces";


MMI_COMMAND MMICommands[]  = 
{
	{ c4, d4, NULL, CDCS_Telnet },
	{ c8, d8, NULL, CDCS_TermServer },
		
	{ c7, d7, NULL, MMI_Trace },
	{ c5, d5, NULL, MMI_Connect },
	{ c6, d6, NULL, MMI_NSLookup },

	{ c9, d9, NULL,	MMI_IfConfig },
	{ c1, d1, NULL, MMI_ShowFreeMem },
	{ c2, d2, NULL,	MMI_ShowThreadState	},
	{ c3, d3, NULL,	MMI_ShowTimers },
	
		
	{ NULL, NULL, NULL, NULL }	// This defines the end of the list
};

// End of MMI Command Definitions

//
// MMI Session handler
//
void DoMMI(NUTDEVICE *stream)
{
	int		l,n;
	u_char	i;
	u_char	buff[128];

	NutPrintString_P(stream, PSTR("\x0C"));
	for(;;) 
	{
		NutPrintString_P(stream, PSTR("\r\nRMU> "));
		NutPrintFlush(stream);
		if(NutDeviceGetLine(stream, buff, sizeof(buff)) < 0)
			break;
		//
		// Ignore blank lines.
		//
		if(buff[0] == 0)
			continue;
		
		// Check for valid commands
		l = strlen(buff);
		for (i = 0; MMICommands[i].cmd != NULL; i++)
		{
			n = strlen_P(MMICommands[i].cmd);
			//NutPrintFormat(stream, "i n = %d %d\r\n",i,n);
			if (l >= n)
			{
				// Only compare the number of characters in each command
				// There is a potential problem here with commands
				// that are a subset of longer commands
				// lets wait and see if we need to fix it
				if(strncmp_P(buff, MMICommands[i].cmd, n) == 0) 
				{
					if (buff[n])									// If arguments supplied
						MMICommands[i].cmd_hdlr(stream, buff+n+1);	// Point at agrument
					else
						MMICommands[i].cmd_hdlr(stream, buff+n);	// Point at NULL
					NutPrintString_P(stream, PSTR(".\r\n"));	
					break;
				}
			}
		}
		// If we processed a valid command 
		if (MMICommands[i].cmd != NULL)	
			continue; // Wait for another one
		
		// Special handler for quit command
		if(strncmp_P(buff, PSTR("quit"), l) == 0) 
		{
			NutPrintString_P(stream, PSTR("\r\n"));
			NutPrintFlush(stream);
			break;
		}

		//
		// Unrecognised command - Display list of valid commands
		//
		NutPrintString_P(stream, PSTR("\r\nCommands\r\n"));
		for (i = 0; MMICommands[i].cmd != NULL; i++)
		{
			NutPrintString_P(stream, MMICommands[i].cmd);
			n = strlen_P(MMICommands[i].cmd);
			n = 10-n;
			while (n--) NutPrintString_P(stream, PSTR(" "));
			NutPrintString_P(stream, PSTR("\t"));
			NutPrintString_P(stream, MMICommands[i].descr);
			NutPrintString_P(stream, PSTR("\r\n"));
		}
			
	}
}


