/*!
 * Copyright (C) 2002 by Call Direct Cellular Solutions Pty. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
  *    This product includes software developed by Call Direct Cellular Solutions Pty. Ltd.
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY CALL DIRECT CELLULAR SOLUTIONS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CALL DIRECT
 * CELLULAR SOLUTIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.calldirect.com.au/
 */

/*!
 * $Log: CDCS_Modem.c,v $
 * Revision 1.1  2002/10/29 12:39:37  harald
 * First check in
 *
 *
 * Revision 0.1  2002/07/30 MJC 
 * Created
 *
 */

#include <string.h>
#include <stdlib.h>
#include <sys/print.h>      /* Formatted output functions. */
#include <sys/ifstream.h>	  /* Stream IO functions */
#include <sys/timer.h>
#include <sys/heap.h>

#include "CDCS_Modem.h"
#include "UltraLite_E47.h"

#ifdef NUTDEBUG
#include <sys/kprint.h>
#include <sys/osdebug.h>
#include <net/netdebug.h>
#endif

#define ATRESPBUFFSIZE		128

int Send_ATCommand(NUTDEVICE *dev, const char *cmd, u_char **resp, u_long timeout)
{
	int		got;
	int		ret;
	u_long	to_in;
	u_long	to;
	int		buf_used;
	int		buf_free;

	// Allocate response buffer
	buf_used = 0;
	buf_free = ATRESPBUFFSIZE;
	if (*resp == 0)
		*resp = NutHeapAlloc(buf_free);

	// Setup read timeout
	to = 1000;
	NutDeviceIOCtl(dev, UART_GETREADTIMEOUT, &to_in);
	NutDeviceIOCtl(dev, UART_SETREADTIMEOUT, &to);

	// Discard anything in the input buffer
	NutDeviceRead (dev,0,0);

	// Send our command
	NutPrintFormat(dev, "%s\r\n", cmd);
	NutPrintFlush(dev);

	do
	{
		// Get the response 
		got = NutIfStreamGetLine(dev, (*resp)+buf_used, buf_free -1);
		if (got == -1)
		{
			// Timeout
			if ((timeout -= 1000) <= 0)
			{
				ret = -1;
				break;
			}
		}
		(*resp)[buf_used+got] = 0;

		// If this is not an echo of the command
		if (strcmp (cmd, *resp))
		{
			buf_used += got;
			buf_free -= got;

			if (buf_free < 2) 
			{
				ret = -1;
				break;
			}
				
			if (strstr (*resp,"OK"))
			{
				ret = 0;
				break;
			}
			if (strstr (*resp,"ERROR"))
			{
				ret = 1;
				break;
			}
		}
	}
	while(1);

	// Reset read timeout
	NutDeviceIOCtl(dev, UART_SETREADTIMEOUT, &to_in);

	return ret;
}

int DisconnectCall (NUTDEVICE *dev)
{
	NutSleep (500);				
	NutPrintString_P(dev, PSTR("+++"));
	NutPrintFlush(dev);
	NutSleep (500);					
	NutPrintString_P(dev, PSTR("ATH\r\n"));
	NutPrintFlush(dev);
	NutSleep (500);
	return 0;
}
