/*!
 * Copyright (C) 2002 by Call Direct Cellular Solutions Pty. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
  *    This product includes software developed by Call Direct Cellular Solutions Pty. Ltd.
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY CALL DIRECT CELLULAR SOLUTIONS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CALL DIRECT
 * CELLULAR SOLUTIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.calldirect.com.au/
 */

/*!
 * $Log: CDCS_Telnet.c,v $
 * Revision 1.1  2002/10/29 12:39:37  harald
 * First check in
 *
 *
 * Revision 0.1  2002/06/20 MJC 
 * Created
 *
 */

#include <string.h>
#include <stdlib.h>
#include <sys/print.h>      /* Formatted output functions. */
#include <sys/ifstream.h>	  /* Stream IO functions */
#include <netinet/sostream.h>
#include <arpa/inet.h>
#include <sys/device.h>

#include "CDCS_MMI.h"
#include "UltraLite_E47.h"
#include "CDCS_Inet.h"

#ifdef NUTDEBUG
#include <sys/kprint.h>
#include <sys/osdebug.h>
#include <net/netdebug.h>
#endif

void CDCS_TermServer(NUTDEVICE *stream, u_char *args)
{
	int			got;
	TCPSOCKET	*sock;
	char		buff[128];
	u_long		to_in;
	u_long		to;
	char		run;
	u_long		ltmp;

	//
	// Send a CR so that the user knows that something is happening
	//
	NutPrintString_P(uart0, PSTR("\r\n"));
	NutPrintFlush(uart0);

	// Setup read timeout and baud rate
	to = 100;
	NutDeviceIOCtl(uart0, UART_GETREADTIMEOUT, &to_in);
	NutDeviceIOCtl(uart0, UART_SETREADTIMEOUT, &to);
	//ltmp = 9600;
    //NutDeviceIOCtl(uart0, UART_SETSPEED, &ltmp);

	
	for(run = 1; run;) 
	{
        // Create a socket.
        sock = NutTcpCreateSocket();
		sock->so_read_to = 10;			// Set 100ms read timeout 

		
		//NutPrintString_P(uart0, PSTR("Listening...\r\n"));
		//NutPrintFlush(uart0);

        // Listen on telnet port. If we return,
        // we got a client.
        NutTcpAccept(sock, 232);

		//NutPrintString_P(uart0, PSTR("Connected...\r\n"));
		//NutPrintFlush(uart0);

		while (1)
		{
			got = NutTcpReceive(sock, buff, sizeof(buff));
			if (got == -1)
				break;
			if (got)
			{
				buff[got] = 0;
				NutPrintString(uart0, buff);
				NutPrintFlush(uart0);
			}

			if ((got = NutIfStreamRead(uart0, buff, sizeof(buff)-1)) > 0)
			{
				buff[got] = 0;
				if (buff[0] == 0x1D)
				{
					run = 0;
					break;
				}
				NutTcpSend(sock, buff, got);
			}
		}

        // Close our socket.
        NutTcpCloseSocket(sock);
    }
	//ltmp = 57600;
    //NutDeviceIOCtl(uart0, UART_SETSPEED, &ltmp);
	NutDeviceIOCtl(uart0, UART_SETREADTIMEOUT, &to_in);
}

void CDCS_Telnet(NUTDEVICE *stream, u_char *args)
{
	int			got;
	TCPSOCKET	*sock;
	char		buff[128];
	u_long		to_in;
	u_long		to;
	int			port;
	char		*p;
	u_long		addr;
	
	//NutKHeapTrace(1);

	//
	// Send a CR so that the user knows that something is happening whilst we resolve the address
	// 
	NutPrintString_P(uart0, PSTR("\r\n"));
	NutPrintFlush(uart0);

	//
	// Get port number if supplied
	// Otherwise use port 23
	//
	p = strchr(args, ' ');
	if (p)
	{
		*p = 0;
		p++;
		if ((port = atoi(p)) == 0)
			port = 23;
	}
	else
		port = 23;


	// 
	// Resolve address
	//
	addr = gethostbyname(args);
	//addr = inet_addr("144.137.125.41");
	if (addr == 0)
	{
		NutPrintFormat(uart0, "ERROR: Gethostbyname timed out");
		NutPrintFlush(uart0);
		return;
	}
	
	//
    // Create a socket.
    //
	sock = NutTcpCreateSocket();	
	sock->so_read_to = 100;			// Set 50ms read timeout 

	// Setup read timeout
	to = 100;
	NutDeviceIOCtl(uart0, UART_GETREADTIMEOUT, &to_in);
	NutDeviceIOCtl(uart0, UART_SETREADTIMEOUT, &to);

	NutPrintFormat(uart0, "Connecting to %s %d Use ^] to quit\r\n", inet_ntoa(addr), port);
	NutPrintFlush(uart0);

	if(NutTcpConnect(sock, addr, port) == 0)
	{
		while (1)
		{
			got = NutTcpReceive(sock, buff, sizeof(buff));
			if (got == -1)
				break;
			if (got)
			{
				buff[got] = 0;
				NutPrintString(uart0, buff);
				NutPrintFlush(uart0);
			}

			if ((got = NutIfStreamRead(uart0, buff, sizeof(buff)-1)) > 0)
			{
				buff[got] = 0;
				if (strchr(buff, 0x1D))
				{
					break;
				}
				NutTcpSend(sock, buff, got);
			}
		}
		NutPrintFormat(uart0, "Done error = %d\r\n", sock->so_last_error);
		NutPrintFlush(uart0);
	}
	else
	{
		NutPrintFormat(uart0, "Connect Failed error = %d \r\n", sock->so_last_error);
		NutPrintFlush(uart0);
	}
	// Close socket
	NutTcpCloseSocket(sock);
	// Reset read timeout
	NutDeviceIOCtl(uart0, UART_SETREADTIMEOUT, &to_in);
}
