/*!
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: rs232d.c,v $
 * Revision 1.5  2002/10/31 16:25:48  harald
 * Mods by troth for Linux
 *
 * Revision 1.3  2002/06/26 17:29:03  harald
 * First pre-release with 2.4 stack
 *
 * Revision 1.2  2002/06/12 11:22:55  harald
 * *** empty log message ***
 *
 * Revision 1.1  2002/06/04 19:00:49  harald
 * First check in
 *
 */

/*!
 * \example rs232d/rs232d.c
 *
 * Simple RS232 server. Use a serial cable to connect the RS232 port
 * of the Ethernut Board with a COM port of a PC. Start a terminal
 * program and a telnet client on the PC. Telnet should connect to
 * the Ethernut Board.
 *
 * Characters typed in the telnet window will appear in the terminal
 * program window and vice versa.
 *
 */

#include <string.h>

#include <dev/nicrtl.h>
#include <dev/uartavr.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>

volatile u_char connected = 0;

TCPSOCKET *sock;
NUTDEVICE *uart;

u_char rxbuff[256];
u_char txbuff[256];

/*
 * From RS232 to socket.
 */
THREAD(Receiver, arg)
{
    int rlen;
    int slen;
    int c;

    for(;;) {
        if(connected)
            if((rlen = NutDeviceRead(uart, rxbuff, sizeof(rxbuff))) > 0)
                for(c = 0; connected && c < rlen; c += slen) 
                    if((slen = NutTcpSend(sock, rxbuff + c, rlen - c)) < 0)
                        connected = 0;
        NutThreadYield();
    }
}

/*
 * From socket to RS232.
 */
void Transmitter(void)
{
    int rlen;
    int slen;
    int c;

    connected = 1;
    while(connected) {
        if((rlen = NutTcpReceive(sock, txbuff, sizeof(txbuff))) < 0)
            break;
        for(c = 0; connected && c < rlen; c += slen) {
            if((slen = NutDeviceWrite(uart, txbuff + c, rlen - c)) < 0)
                break;
            NutDeviceWrite(uart, 0, 0);
        }
        NutThreadYield();
    }
    connected = 0;
}

/*
 * Main application routine. 
 *
 * Nut/OS automatically calls this entry after initialization.
 */
THREAD(NutMain, arg)
{
    u_long baud = 9600;

    NutRegisterDevice(&devUart0, 0, 0);
    uart = NutDeviceOpen("uart0");
    NutDeviceIOCtl(uart, UART_SETSPEED, &baud);

    /*
     * Register Realtek controller at address 8300 hex
     * and interrupt 5.
     */
    NutRegisterDevice(&devEth0, 0x8300, 5);

    /*
     * Configure lan interface. 
     *
     */
    NutNetAutoConfig("eth0");
    NutThreadCreate("xmit", Receiver, 0, 1024);

    /*
     * Now loop endless for connections.
     */
    for(;;) {
        /*
         * Create a socket.
         */
        sock = NutTcpCreateSocket();

        /*
         * Listen on telnet port. If we return,
         * we got a client.
         */
        NutTcpAccept(sock, 23);

        /*
         * Call RS232 transmit routine.
         */
        Transmitter();

        /*
         * Close our socket.
         */
        NutTcpCloseSocket(sock);
    }
}
