/*!
 * Copyright (C) 2002 by Call Direct Cellular Solutions Pty. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
  *    This product includes software developed by Call Direct Cellular Solutions Pty. Ltd.
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY CALL DIRECT CELLULAR SOLUTIONS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CALL DIRECT
 * CELLULAR SOLUTIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.calldirect.com.au/
 */

/*
 * $Log: PPP0.c,v $
 * Revision 1.1  2002/10/29 12:44:44  harald
 * PPP support added
 *
 *
 * Revision 1.0  2002/03/28 MJC CDCS
 * New device structure implemented
 *
 */

#include <net/if_var.h>
#include <sys/uart.h>
#include <net/PPPif.h>

PPPINFO dcb_PPP0;

/*!
 * \brief Network interface information structure.
 *
 */
IFNET ifn_PPP0 = {
    IFT_PPP,		/*!< \brief Interface type. */
    {0,0,0,0,0,0},  /*!< \brief Hardware net address. */
	0,				/*!< \brief Host device for PPP interfaces*/
    0,              /*!< \brief IP address. */
    0,              /*!< \brief Remote IP address for point to point. */
    0,              /*!< \brief IP network mask. */
	0,				/*!< \brief IP Primary DNS */
	0,				/*!< \brief IP Secondary DNS*/
    PPP_MRU,        /*!< \brief Maximum size of a transmission unit. */
    0,              /*!< \brief Packet identifier. */
    0,              /*!< \brief Linked list of arp entries. */
    0,              /*!< \brief Routine to pass received data to. */
    PPPOutput		/*!< \brief Output routine. */
};

/*!
 * \brief Device information structure.
 *
 * Applications must pass this structure to NutRegisterDevice() 
 * to bind this Ethernet device driver to the Nut/OS kernel.
 * Having done that, the application may call NutNetIfConfig()
 * with the name \em PPP0 of this driver to initialize the network
 * interface.
 * 
 */

NUTDEVICE devPPP0 = {
    0,              /*!< Pointer to next device. */
    { 'P', 'P', 'P', '0', 0, 0, 0, 0, 0 },    /*!< Unique device name. */
    IFTYP_NET,      /*!< Type of device. */
    0,              /*!< Base address. */
    0,              /*!< First interrupt number. */
    &ifn_PPP0,      /*!< Interface control block. */
    &dcb_PPP0,      /*!< Driver control block. */
    PPPInit,        /*!< Driver initialization routine. */
    0               /*!< Driver specific control function. */
};

/*@}*/
