#ifndef _FS_UROMFS_H_
#define _FS_UROMFS_H_

/*
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * $Log: uromfs.h,v $
 * Revision 1.6  2002/11/02 15:17:00  harald
 * Library dependencies moved to compiler.h
 *
 * Revision 1.5  2002/07/03 16:45:40  harald
 * Using GCC 3.2
 *
 * Revision 1.4  2002/06/26 17:29:17  harald
 * First pre-release with 2.4 stack
 *
 */
#include <sys/types.h>

/*!
 * \file fs/uromfs.h
 * \brief Micro-ROM file system definitions.
 */

/*!
 * \brief Mikro-ROM directory entry type.
 */
typedef struct _ROMENTRY ROMENTRY;

/*!
 * \struct _ROMENTRY uromfs.h fs/uromfs.h
 * \brief Mikro-ROM directory entry structure.
 */
#ifdef __IMAGECRAFT__
struct _ROMENTRY {
    ROMENTRY *rome_next;    /*!< Link to next ROMENTRY structure. */
    u_char   *rome_name;    /*!< Filename. */
    u_short   rome_size;    /*!< File size. */
    const char *rome_data;  /*!< File data. */
};
#else
struct _ROMENTRY {
    ROMENTRY *rome_next;    /*!< Link to next ROMENTRY structure. */
    u_char   *rome_name;    /*!< Filename. */
    u_short   rome_size;    /*!< File size. */
    prog_char *rome_data; /* __attribute__ ((progmem));  !< File data. */
};
#endif

/*!
 * \brief Mikro-ROM file information type.
 */
typedef struct _ROMFILE ROMFILE;

/*!
 * \struct _ROMFILE uromfs.h fs/uromfs.h
 * \brief Mikro-ROM file information structure.
 */
struct _ROMFILE {
    ROMENTRY *romf_entry;   /*!< Points to ROMENTRY */
    u_short  romf_pos;      /*!< Current read position. */
};

/*!
 * \brief Linked list of all microROM files.
 */
extern ROMENTRY *romEntryList;

extern ROMFILE *NutRomFileOpen(char *name);
extern int NutRomFileClose(ROMFILE *romf);
extern int NutRomFileRead(ROMFILE *romf, void *data, size_t size);
extern int NutRomFileSize(ROMFILE *romf);
extern int NutRomFileSeek(ROMFILE *romf, int pos);

#endif
