/*
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

/*
 * $Log: osdebug.c,v $
 * Revision 1.6  2002/06/26 17:29:44  harald
 * First pre-release with 2.4 stack
 *
 */

#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/heap.h>
#include <sys/atom.h>
#include <sys/print.h>

char os_trace = 0;
char heap_trace = 0;

static char *states[] = { "TRM", "RUN", "RDY", "SLP" };

void NutKDumpThreadQueue(NUTTHREADINFO *tdp)
{
    NutEnterCritical();
    NutPrintString_P(0, PSTR("\r\nHndl Name     Prio Sta QUE  Timr StkP FreeMem\r\n"));
    /*                          1234 12345678 1234 123 1234 1234 1234 12345 */
    if(tdp == SIGNALED) {
        NutPrintString_P(0, PSTR("SIGNALED\r\n"));
        NutExitCritical();
        return;
    }
    while(tdp) {
        NutPrintFormat(0, "%04X %-8s %4u %s %04X %04X %04X %5u %s\r\n", 
                        tdp, tdp->td_name, tdp->td_priority,
                        states[tdp->td_state], tdp->td_queue, tdp->td_timer,
                        tdp->td_sp, (u_short)tdp->td_sp - (u_short)tdp->td_memory,
                        *((u_long *)tdp->td_memory) != DEADBEEF &&
                        *((u_long *)(tdp->td_memory + 4)) != DEADBEEF &&
                        *((u_long *)(tdp->td_memory + 8)) != DEADBEEF &&
                        *((u_long *)(tdp->td_memory + 12)) != DEADBEEF 
                        ? "Corr" : "OK");
        tdp = tdp->td_qnxt;
    }
    NutExitCritical();
}

void NutKDumpThreadList(void)
{
    NUTTHREADINFO *tqp;
    NUTTHREADINFO *tdp;

    NutEnterCritical();
    tdp = nutThreadList;
    NutPrintString_P(0, PSTR("\r\nHndl Name     Prio Sta Que  Timr StkP FreeMem\r\n"));
    /*                   1234 12345678 1234 123 1234 1234 1234 12345 */
    while(tdp) {
        NutPrintFormat(0, "%04X %-8s %4u %s %04X %04X %04X %5u %s", 
                        tdp, tdp->td_name, tdp->td_priority,
                        states[tdp->td_state], tdp->td_queue, tdp->td_timer,
                        tdp->td_sp, (u_short)tdp->td_sp - (u_short)tdp->td_memory,
                        *((u_long *)tdp->td_memory) != DEADBEEF ? "CR" : "OK");
        if(tdp->td_queue) {
            tqp = *(NUTTHREADINFO **)(tdp->td_queue);
            if(tqp == SIGNALED)
                NutPrintString_P(0, PSTR("SIGNALED"));
            else {
                while(tqp) {
                    NutPrintFormat(0, " %04X", tqp);
                    tqp = tqp->td_qnxt;
                }
            }
        }
        NutPrintString(0, "\r\n");
        tdp = tdp->td_next;
    }
    NutExitCritical();
}

void NutKDumpTimerList(void)
{
    NUTTIMERINFO *tnp;

    NutEnterCritical();
    if((tnp = nutTimerList) != 0) {
        NutPrintString_P(0, PSTR("Addr Ticks  Left Callback\r\n"));
        while(tnp) {
            NutPrintFormat(0, "%04X%6lu%6lu ", tnp, tnp->tn_ticks, tnp->tn_ticks_left);
            if(tnp->tn_callback == NutThreadWake)
                NutPrintString_P(0, PSTR("NutThreadWake"));
            else if(tnp->tn_callback == NutEventTimeout)
                NutPrintString_P(0, PSTR("NutEventTimeout"));
            else
                NutPrintFormat(0, "%05lX", (u_long)((u_short)tnp->tn_callback) << 1);
            NutPrintFormat(0, "(%04X)\r\n", tnp->tn_arg);
            tnp = tnp->tn_next;
        }
    }
    if((tnp = nutTimerPool) != 0) {
        NutPrintString_P(0, PSTR("Pool:"));
        while(tnp) {
            NutPrintFormat(0, " %04X", tnp);
            tnp = tnp->tn_next;
        }
        NutPrintString(0, "\r\n");
    }
    NutExitCritical();
}

/*!
 * \brief Switch on/off OS trace.
 */
void NutKOsTrace(char enable)
{
    os_trace = enable;
}

void NutKDumpHeap(void)
{
    HEAPNODE *node;
    u_short sum = 0;
    u_short avail;

    NutPrintString(0, "\r\n");
    for(node = heapFreeList; node; node = node->hn_next) {
        sum += node->hn_size;
        NutPrintFormat(0, "%04x %5d\r\n", (u_short)node, node->hn_size);
        if((u_short)node < 0x60 || (u_short)node > 0x7fff)
            break;
    }
    if((avail = NutHeapAvailable()) != sum)
        NutPrintFormat(0, "%u counted, but %u reported\r\n", sum, avail);
    else
        NutPrintFormat(0, "%u bytes free\r\n", avail);
}


/*!
 * \brief Switch on/off dynamic memory trace.
 */
void NutKHeapTrace(char enable)
{
    heap_trace = enable;
}



